<?php

/**
 * @file
 * Main pages file for Gift Exchange module.
 */

/**
 * Show the Gift Exchange as form.
 */
function gift_exchange_form($node) {
  // Add ctools ajax libraries.
  ctools_include('modal');
  ctools_include('ajax');
  ctools_modal_add_js();
  
  global $user;
  $form = array();

  // Hook before party page is built.
  $form_altered = module_invoke_all('ge_party_prebuild', $form, $node);
  $form = $form_altered != NULL ? $form = $form_altered : $form;

  // Load functions.
  module_load_include('inc', 'gift_exchange', 'includes/gift_exchange.functions');

  // Get counts.
  $members_count = gift_exchange_get_party_members_count($node->nid);
  $exceptions_count = gift_exchange_get_party_exceptions_count($node->nid);
  $connections_count = gift_exchange_get_party_connections_count($node->nid);

  // Breadcrumb.
  $breadcrumb = array();
  $breadcrumb[] = l(t('My Parties'), 'gift-exchange/my-parties');
  $breadcrumb[] = $node->title;
  gift_exchange_breadcrumb($breadcrumb);

  // Set page title.
  drupal_set_title($node->title);

  // Get party status.
  $status = gift_exchange_get_party_status($node->nid);

  // Main container.
  $form['main_container'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'main_container',
      'class' => array('main-container', $status),
    ),
  );

  // Actions container.
  $form['main_container']['actions'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'actions_container',
      'class' => array('actions-container', 'clearfix'),
    ),
    '#weight' => 0,
  );

  // Info container.
  $form['main_container']['info'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'info_container',
      'class' => array('clearfix', 'info-container'),
    ),
    '#weight' => 1,
  );

  // Tabs container.
  $form['main_container']['tabs'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'tabs_container',
      'class' => array('clearfix', 'tabs-container'),
    ),
    '#weight' => 2,
  );

  // Add members.
  $form['main_container']['actions']['add_members'] = array(
    '#type' => 'html_tag',
    '#tag' => 'div',
    '#value' => l(t('Add members'), 'gift-exchange/my-parties/' . $node->nid . '/add-members',
      array(
        'attributes' => array(
          'class' => array('action', 'add', 'add-members'),
        )
      )
    ),
    '#attributes' => array(
      'class' => array('action-container', 'add-container'),
    ),
  );

  // Add exceptions.
  $form['main_container']['actions']['add_exceptions'] = array(
    '#type' => 'html_tag',
    '#tag' => 'div',
    '#value' => l(t('Add exceptions'), 'gift-exchange/my-parties/' . $node->nid . '/add-exceptions',
      array(
        'attributes' => array(
          'class' => array('action', 'add', 'add-exceptions'),
        )
      )
    ),
    '#attributes' => array(
      'class' => array('action-container', 'add-container'),
    ),
  );

  // Process.
  $process_party = ctools_modal_text_button(
    t('Process'),
    'gift-exchange/confirm/nojs/' . $node->nid . '/process',
    t('Process party'),
    'ctools-modal-modal-popup-small action process');
  $form['main_container']['actions']['process'] = array(
    '#type' => 'html_tag',
    '#tag' => 'div',
    '#value' => $process_party,
    '#attributes' => array(
      'class' => array('action-container', 'process-container'),
    ),
  );

  // Reset.
  $reset_party = ctools_modal_text_button(
    t('Reset'),
    'gift-exchange/confirm/nojs/' . $node->nid . '/reset',
    t('Reset party'),
    'ctools-modal-modal-popup-small action reset');
  $form['main_container']['actions']['reset'] = array(
    '#type' => 'html_tag',
    '#tag' => 'div',
    '#value' => $reset_party,
    '#attributes' => array(
      'class' => array('action-container', 'reset-container'),
    ),
  );

  // View as member.
  if (gift_exchange_get_member_email_is_in_party($user->mail, $node->nid)) {
    $form['main_container']['actions']['member-view'] = array(
      '#type' => 'html_tag',
      '#tag' => 'div',
      '#value' => l(t('View as member'), 'gift-exchange/my-parties/' . $node->nid . '/member',
        array(
          'attributes' => array(
            'class' => array('action', 'view-as-member'),
          )
        )
      ),
      '#attributes' => array(
        'class' => array('action-container', 'view-as-member-container'),
      ),
    );
  }

  // Edit.
  if ($status == 'setup' || $status == 'unsolvable') {
    $form['main_container']['actions']['edit'] = array(
      '#type' => 'html_tag',
      '#tag' => 'div',
      '#value' => l(t('Edit'), 'gift-exchange/my-parties/' . $node->nid . '/edit',
        array(
          'attributes' => array(
            'class' => array('action', 'edit'),
          )
        )
      ),
      '#attributes' => array(
        'class' => array('action-container', 'edit-container'),
      ),
    );
  }

  // Headers container. Themed as item list to show them as jquery tabs.
  $headers = array();

  if ($members_count) {
    // Members container.
    $form['main_container']['tabs']['members'] = array(
      '#type' => 'container',
      '#attributes' => array(
        'id' => 'members_container',
        'class' => array('clearfix'),
      ),
      '#weight' => 1,
    );

    // Members Header.
    $headers[] = l(t('Members'), '', array(
      'fragment' => 'members_container',
      'external' => TRUE,
      'attributes' => array(
        'title' => t('Members'),
      ),
    ));

    // Members View.
    $form['main_container']['tabs']['members']['view'] = array(
      '#type' => 'item',
      '#markup' => views_embed_view('ge_members', 'block', $node->nid),
    );
  }

  if ($exceptions_count) {
    // Exceptions container.
    $form['main_container']['tabs']['exceptions'] = array(
      '#type' => 'container',
      '#attributes' => array(
        'id' => 'exceptions_container',
        'class' => array('clearfix'),
      ),
      '#weight' => 2,
    );

    // Exceptions Header.
    $headers[] = l(t('Exceptions'), '', array(
      'fragment' => 'exceptions_container',
      'external' => TRUE,
      'attributes' => array(
        'title' => t('Exceptions'),
      ),
    ));

    // Exceptions View.
    $form['main_container']['tabs']['exceptions']['view'] = array(
      '#type' => 'item',
      '#markup' => views_embed_view('ge_exceptions', 'block', $node->nid),
    );
  }

  if (($status == 'setup'
    || $status == 'processed' || $status == 'sending_mail'
    || $status == 'archived') && $connections_count) {
    // Connections container.
    $form['main_container']['tabs']['connections'] = array(
      '#type' => 'container',
      '#attributes' => array(
        'id' => 'connections_container',
        'class' => array('clearfix'),
      ),
      '#weight' => 3,
    );

    // Connections Header.
    $headers[] = l(t('Connections'), '', array(
      'fragment' => 'connections_container',
      'external' => TRUE,
      'attributes' => array(
        'title' => t('Connections'),
      ),
    ));

    // Connections View.
    $form['main_container']['tabs']['connections']['view'] = array(
      '#type' => 'item',
      '#markup' => views_embed_view('ge_connections', 'block', $node->nid),
    );
  }

  // Show elements according to status.
  if ($status == 'setup') {
    $info_header = t('Add members and exceptions and click "Process".');
    $form['main_container']['actions']['reset']['#access'] = FALSE;
    $form['main_container']['actions']['member-view']['#access'] = FALSE;
    $form['main_container']['tabs']['connections']['#access'] = FALSE;
  }
  elseif ($status == 'processing') {
    $info_header = t('Gift Generator is processing Members and Exceptions. Check again in a moment...');
    $info_percent_done = intval(100 * $connections_count / $members_count / (int)$node->field_ge_gifts[LANGUAGE_NONE][0]['value']);
    gift_exchange_auto_refresh(60);
    $form['main_container']['actions']['add_members']['#access'] = FALSE;
    $form['main_container']['actions']['add_exceptions']['#access'] = FALSE;
    $form['main_container']['actions']['process']['#access'] = FALSE;
    $form['main_container']['actions']['member-view']['#access'] = FALSE;
  }
  elseif ($status == 'sending_mail') {
    $info_header = t('We are sending the mails...');
    $connections_count_sent = gift_exchange_get_party_connections_count($node->nid, TRUE);
    $info_percent_done = intval(100 * $connections_count_sent/$connections_count);
    gift_exchange_auto_refresh(60);
    $form['main_container']['actions']['add_members']['#access'] = FALSE;
    $form['main_container']['actions']['add_exceptions']['#access'] = FALSE;
    $form['main_container']['actions']['process']['#access'] = FALSE;
  }
  elseif ($status == 'processed') {
    $info_header = t('Party is processed. Exchange date is due to !date',
      array('!date' => format_date($node->field_ge_exchange_date[LANGUAGE_NONE][0]['value'])));
    $form['main_container']['actions']['add_members']['#access'] = FALSE;
    $form['main_container']['actions']['add_exceptions']['#access'] = FALSE;
    $form['main_container']['actions']['process']['#access'] = FALSE;
  }
  elseif ($status == 'reset') {
    $info_header = t('Party is being reset. Check again in a moment...');
    $info_percent_done = (int)((1 - ($connections_count / ($members_count * (int)$node->field_ge_gifts[LANGUAGE_NONE][0]['value']))) * 100);
    gift_exchange_auto_refresh(60);
    $form['main_container']['actions']['add_members']['#access'] = FALSE;
    $form['main_container']['actions']['add_exceptions']['#access'] = FALSE;
    $form['main_container']['actions']['process']['#access'] = FALSE;
    $form['main_container']['actions']['reset']['#access'] = FALSE;
    $form['main_container']['actions']['member-view']['#access'] = FALSE;
    $form['main_container']['tabs']['connections']['#access'] = FALSE;
  }
  elseif ($status == 'unsolvable_reset') {
    $info_header = t('Party is being reset because it is unsolvable. Try to remove some exceptions and/or reduce gifts per member when it is done resetting...');
    $info_percent_done = (int)((1 - ($connections_count / ($members_count * (int)$node->field_ge_gifts[LANGUAGE_NONE][0]['value']))) * 100);
    gift_exchange_auto_refresh(60);
    $form['main_container']['actions']['add_members']['#access'] = FALSE;
    $form['main_container']['actions']['add_exceptions']['#access'] = FALSE;
    $form['main_container']['actions']['process']['#access'] = FALSE;
    $form['main_container']['actions']['reset']['#access'] = FALSE;
    $form['main_container']['actions']['member-view']['#access'] = FALSE;
    $form['main_container']['tabs']['connections']['#access'] = FALSE;
  }
  elseif ($status == 'unsolvable') {
    $info_header = t('Party is unsolvable. Try to remove some exceptions and/or reduce gifts per member and click process.');
    $form['main_container']['actions']['reset']['#access'] = FALSE;
    $form['main_container']['actions']['member-view']['#access'] = FALSE;
    $form['main_container']['tabs']['connections']['#access'] = FALSE;
  }
  elseif ($status == 'archived') {
    $info_header = t('Party is archived. If you want you may clone it or start a new one.');
    $form['main_container']['actions']['add_members']['#access'] = FALSE;
    $form['main_container']['actions']['add_exceptions']['#access'] = FALSE;
    $form['main_container']['actions']['process']['#access'] = FALSE;
    $form['main_container']['actions']['reset']['#access'] = FALSE;
    $form['main_container']['actions']['member-view']['#access'] = FALSE;
  }

  // Info header.
  $form['main_container']['info']['header'] = array(
    '#type' => 'html_tag',
    '#value' => $info_header,
    '#tag' => 'h3',
    '#attributes' => array('class' => array('header', 'info', $status)),
  );

  // Percent done.
  if (isset($info_percent_done)) {
    $form['main_container']['info']['percent_done'] = array(
      '#type' => 'html_tag',
      '#value' => ($info_percent_done == 100 ? 99 : $info_percent_done) . '% done',
      '#tag' => 'h3',
      '#attributes' => array('class' => array('header', 'percent_done')),
    );
  }

  // Create jquery tabs.
  if ($members_count || $exceptions_count || $connections_count) {
    // Add js.
    drupal_add_library ( 'system' , 'ui.tabs' );
    drupal_add_js('(function($) { $( "#tabs_container" ).tabs(); })(jQuery);',
      array('type' => 'inline', 'scope' => 'footer', 'weight' => 5)
    );

    // Add headers.
    $form['main_container']['tabs']['headers'] = array (
      '#items' => $headers,
      '#theme' => 'item_list',
      '#attributes' => array(
        'id' => 'tabs_headers',
        'class' => array('clearfix'),
      ),
      '#weight' => 0,
    );
  }

  // Back.
  $form['back'] = array(
    '#type' => 'html_tag',
    '#value' => l(t('Back'), 'gift-exchange/my-parties',
      array(
        'attributes' => array(
          'class' => array('action', 'back'),
        ),
      )
    ),
    '#tag' => 'div',
    '#attributes' => array(
      'class' => array('back-container', 'clearfix')
    ),
  );

  // Hook to alter party view form.
  $form_altered = module_invoke_all('ge_party_view_alter', $form, $node);
  $form = $form_altered != NULL ? $form_altered : $form;

  return $form;
}

/**
 * Callback for process.
 *
 * @param $node
 *   Gift Exchange Party.
 */
function gift_exchange_party_process_callback($node) {
  // Load functions.
  module_load_include('inc', 'gift_exchange', 'includes/gift_exchange.functions');

  // Check Exchange date.
  if (!isset($node->field_ge_exchange_date[LANGUAGE_NONE][0]['value'])) {
    drupal_set_message(t('Please set an exchange date.'), 'warning');
    drupal_goto('gift-exchange/my-parties/' . $node->nid . '/edit');
    exit;
  }

  $members = gift_exchange_get_party_members_count($node->nid);
  $gifts = $node->field_ge_gifts[LANGUAGE_NONE][0]['value'];

  if ($members < 3) {
    drupal_set_message(t('Party needs at least 3 members.'), 'warning');
    drupal_goto('gift-exchange/my-parties/' . $node->nid);
    exit;
  }
  elseif ($gifts >= $members) {
    drupal_set_message(t('Please set a lower number of gifts per member. (Maximum: members minus one)'), 'warning');
    drupal_goto('gift-exchange/my-parties/' . $node->nid . '/edit');
    exit;
  }

  gift_exchange_set_party_status($node->nid, 'processing');
  drupal_goto('gift-exchange/my-parties/' . $node->nid);
}

/**
 * Callback for reset.
 *
 * @param $node
 *   Gift Exchange Party.
 */
function gift_exchange_party_reset_callback($node) {
  // Load functions.
  module_load_include('inc', 'gift_exchange', 'includes/gift_exchange.functions');
  gift_exchange_set_party_status($node->nid, 'reset');
  drupal_goto('gift-exchange/my-parties/' . $node->nid);
}

/**
 * Show the Gift Exchange parties and some actions.
 */
function gift_exchange_my_parties_form() {
  // Set page title.
  drupal_set_title(t('My Parties'));

  // Load functions.
  $file_path = module_load_include('inc', 'gift_exchange', 'includes/gift_exchange.functions');

  // Breadcrumb.
  $breadcrumb = array();
  $breadcrumb[] = t('My Parties');
  gift_exchange_breadcrumb($breadcrumb);

  // Add ctools ajax libraries.
  ctools_include('modal');
  ctools_include('ajax');
  ctools_modal_add_js();

  $form = array();

  $form['#attributes']['id'] = 'gift-exchange-my-parties-form';
  $form['#attributes']['class'][] = 'gift-exchange-my-parties-form';

  // Main container.
  $form['main_container'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'main_container',
      'class' => array('clearfix'),
    ),
  );

  // Actions container.
  $form['main_container']['actions'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'actions_container',
      'class' => array('clearfix', 'actions-container'),
    ),
  );

  // New party.
  $form['main_container']['actions']['new'] = array(
    '#type' => 'html_tag',
    '#tag' => 'div',
    '#value' => l(t('New Party'), 'gift-exchange/my-parties/new',
      array(
        'attributes' => array(
          'class' => array('action', 'add'),
        )
      )
    ),
    '#attributes' => array(
      'class' => array('action-container'),
    ),
  );

  // View.
  $form['main_container']['view'] = array(
    '#type' => 'item',
    '#markup' => views_embed_view('ge_my_parties', 'block'),
  );

  return $form;
}

/**
 * New party form.
 *
 * @return object
 *   New form.
 */
function gift_exchange_new_form() {

  // Set page title.
  drupal_set_title(t('New Gift Exchange Party'));

  // Load functions.
  $file_path = module_load_include('inc', 'gift_exchange', 'includes/gift_exchange.functions');

  // Breadcrumb.
  $breadcrumb = array();
  $breadcrumb[] = l(t('My Parties'), 'gift-exchange/my-parties');
  $breadcrumb[] = t('New');
  gift_exchange_breadcrumb($breadcrumb);

  module_load_include('inc', 'node', 'node.pages');
  $form = node_add('gift_exchange');

  // Back.
  $form['actions']['back'] = array(
    '#type' => 'html_tag',
    '#value' => l(t('Back'), 'gift-exchange/my-parties',
      array(
        'attributes' => array(
          'class' => array('action', 'back'),
        ),
      )
    ),
    '#tag' => 'div',
    '#attributes' => array(
      'class' => array('back-container')
    ),
  );

  return $form;
}

/**
 * Edit party form.
 *
 * @return object
 *   Edit form.
 */
function gift_exchange_edit_form($node) {

  // Set page title.
  drupal_set_title(t('Edit') . ' -> ' . $node->title);

  // Load functions.
  $file_path = module_load_include('inc', 'gift_exchange', 'includes/gift_exchange.functions');

  $status = gift_exchange_get_party_status($node->nid);

  // Breadcrumb.
  $breadcrumb = array();
  $breadcrumb[] = l(t('My Parties'), 'gift-exchange/my-parties');
  $breadcrumb[] = l($node->title, 'gift-exchange/my-parties/' . $node->nid);
  $breadcrumb[] = t('Edit');
  gift_exchange_breadcrumb($breadcrumb);

  module_load_include('inc', 'node', 'node.pages');
  $form = drupal_get_form('gift_exchange_node_form', $node);

  // Main container.
  $form['main_container'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'main_container',
      'class' => array($status),
    ),
    '#weight' => -10,
  );

  // Actions container.
  $form['main_container']['actions'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'actions_container',
      'class' => array('actions-container', 'clearfix'),
    ),
    '#weight' => 0,
  );

  $form['main_container']['actions']['view'] = array(
    '#type' => 'html_tag',
    '#tag' => 'div',
    '#value' => l(t('View party'), 'gift-exchange/my-parties/' . $node->nid,
      array(
        'attributes' => array(
          'class' => array('action', 'view'),
        )
      )
    ),
    '#attributes' => array(
      'class' => array('action-container', 'view-container'),
    ),
  );

  return $form;
}

/**
 * Submit handler on new Party save. Redirect.
 */
function gift_exchange_new_form_submit($form, &$form_state) {
  $form_state['redirect'] = 'gift-exchange/my-parties/' . $form_state['nid'];
}

/**
 * Add Members form.
 *
 * @return object
 *   Import form.
 */
function gift_exchange_add_members_form($form, &$form_state, $node) {

  // Set page title.
  drupal_set_title(t('Add members') . ' -> ' . $node->title);

  // Load functions.
  $file_path = module_load_include('inc', 'gift_exchange', 'includes/gift_exchange.functions');

  // Breadcrumb.
  $breadcrumb = array();
  $breadcrumb[] = l(t('My Parties'), 'gift-exchange/my-parties');
  $breadcrumb[] = l($node->title, 'gift-exchange/my-parties/' . $node->nid);
  $breadcrumb[] = t('Add Members');
  gift_exchange_breadcrumb($breadcrumb);

  // Getting node id for validate and submit.
  $form_state['party_nid'] = $node->nid;

  // Main container.
  $form['main_container'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'main_container',
      'class' => array('main-container'),
    ),
  );

  // Actions container.
  $form['main_container']['actions'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'actions_container',
      'class' => array('actions-container', 'clearfix'),
    ),
    '#weight' => 0,
  );

  $form['main_container']['actions']['view'] = array(
    '#type' => 'html_tag',
    '#tag' => 'div',
    '#value' => l(t('View party'), 'gift-exchange/my-parties/' . $node->nid,
      array(
        'attributes' => array(
          'class' => array('action', 'view-as-admin'),
        )
      )
    ),
    '#attributes' => array(
      'class' => array('action-container', 'view-as-admin-container'),
    ),
  );

  // Because we have many fields with the same values, we have to set
  // #tree to be able to access them.
  $form['#tree'] = TRUE;
  $form['main_container']['members'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'members_container',
      'class' => array('members-container', 'clearfix'),
    ),
  );

  // Build the fieldset with the proper number of names. We'll use
  // $form_state['num_members'] to determine the number of textfields to build.
  if (empty($form_state['num_members'])) {
    $form_state['num_members'] = 5;
  }
  for ($i = 0; $i < $form_state['num_members']; $i++) {
    // Get if even or odd.
    $row_class = $i % 2 == 0 ? 'even' : 'odd';

    $form['main_container']['members']['member_' . $i] = array(
      '#type' => 'container',
      '#attributes' => array(
        'id' => 'member_container',
        'class' => array('member-container', 'clearfix', $row_class),
      ),
    );
    $form['main_container']['members']['member_' . $i]['name'] = array(
      '#type' => 'textfield',
      '#title' => t('Name !num', array('!num' => $i+1)),
    );
    $form['main_container']['members']['member_' . $i]['email'] = array(
      '#type' => 'textfield',
      '#title' => t('Email !num', array('!num' => $i+1)),
    );

    // If no members, prompt user to add his name too.
    $members_count = gift_exchange_get_party_members_count($node->nid);
    if ($members_count == 0
      && !isset($form['main_container']['members']['member_' . $i]['name']['#value'])) {
      global $user;
      // @todo: Fix it.
      $form['main_container']['members']['member_0']['name']['#defaut'] = $user->name;
      $form['main_container']['members']['member_0']['email']['#defaut'] = $user->mail;
    }
  }

  // Add member.
  $form['main_container']['add_member'] = array(
    '#type' => 'submit',
    '#value' => t('Add one more'),
    '#submit' => array('_gift_exchange_add_members_form_add_one_submit'),
    '#ajax' => array(
      'callback' => '_gift_exchange_add_members_form_add_one_callback',
      'wrapper' => 'members_container',
    ),
   '#attributes' => array(
      'class' => array('add-button'),
    ),
  );

  // Actions container.
  $form['actions'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'class' => array('actions', 'clearfix'),
    ),
  );

  // Back.
  $form['actions']['back'] = array(
    '#type' => 'html_tag',
    '#value' => l(t('Back'), 'gift-exchange/my-parties/' . $node->nid,
      array(
        'attributes' => array(
          'class' => array('action', 'back'),
        ),
        'fragment' => 'members_container',
      )
    ),
    '#tag' => 'div',
    '#attributes' => array(
      'class' => array('back-container', 'clearfix')
    ),
  );
  // Submit.
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#ajax' => array(
      'callback' => '_gift_exchange_add_members_form_callback',
      'wrapper' => 'members_container',
    ),
    '#validate' => array('_gift_exchange_add_members_form_validate_callback'),
    '#attributes' => array(
      'class' => array('save-button'),
    ),
  );


  return $form;
}

/**
 * Callback for both ajax-enabled buttons.
 *
 * Selects and returns the container.
 */
function _gift_exchange_add_members_form_add_one_callback($form, &$form_state) {
  return $form['main_container']['members'];
}

/**
 * Submit handler for the "add-one-more" button.
 *
 * Increments the max counter and causes a rebuild.
 */
function _gift_exchange_add_members_form_add_one_submit($form, &$form_state) {
  $form_state['num_members']++;
  $form_state['rebuild'] = TRUE;
}

/**
 * Add Members validate form.
 */
function _gift_exchange_add_members_form_validate_callback($form, &$form_state) {
  // Load functions.
  module_load_include('inc', 'gift_exchange', 'includes/gift_exchange.functions');
  
  // Check names/emails are correct.
  if (isset($form_state['values']['main_container']['members'])) {
    foreach ($form_state['values']['main_container']['members'] as $key => $member) {
      
      // Get some helper values.
      $name = isset($form_state['values']['main_container']['members'][$key]['name']) ?
        $form_state['values']['main_container']['members'][$key]['name'] : '';
      $name_len = strlen($name);
      $name_title = $form['main_container']['members'][$key]['name']['#title'];
      $email = isset($form_state['values']['main_container']['members'][$key]['email']) ?
        $form_state['values']['main_container']['members'][$key]['email'] : '';
      $email_len = strlen($email);
      $email_title = $form['main_container']['members'][$key]['email']['#title'];
      $is_empty = $name_len == 0 && $email_len == 0;
   
      // If both are empty skip.
      
      if (!$is_empty) {
        // Check name.
        if ($name_len == 0) {
          form_set_error('main_container][members][' . $key . '][name',
            t('Name is required at %title.',
              array('%title' => $name_title)));
        }
        
        if ($email_len == 0 || !valid_email_address($email)) {
          form_set_error('main_container][members][' . $key . '][email',
            t('Email is not valid at %title.',
              array('%title' => $email_title)));
        }
        
        $member_exists = gift_exchange_get_member_name_email_is_in_party($name, $email, $form_state['party_nid']);
        if ($member_exists) {
          form_set_error('main_container][members][' . $key . '][name',
                         t('Member %title already in party.',
                           array('%title' => $name_title)));
          form_set_error('main_container][members][' . $key . '][email');
        }
      }
    }
  }
}

/**
 * Add Members ajax callback.
 */
function _gift_exchange_add_members_form_callback($form, &$form_state) {
  if (form_get_errors()) {
    $form_state['rebuild'] = TRUE;
    return $form['main_container']['members'];
  }

  // Add Members submit form.
  $response = _gift_exchange_add_members_form_submit($form, $form_state);
  return $response;
}

/**
 * Add Members submit form.
 */
function _gift_exchange_add_members_form_submit($form, &$form_state) {
  
  module_load_include('inc', 'gift_exchange', 'includes/gift_exchange.functions');
  
  // Members that will be added.
  $num_members = 0;
  
  // Load Party.
  $party = node_load($form_state['party_nid']);

  // Create members.
  if (isset($form_state['values']['main_container']['members'])) {
    foreach ($form_state['values']['main_container']['members'] as $key => $member) {
      // Get some helper values.
      $name = isset($form_state['values']['main_container']['members'][$key]['name']) ?
        $form_state['values']['main_container']['members'][$key]['name'] : '';
      $name_len = strlen($name);
      $email = isset($form_state['values']['main_container']['members'][$key]['email']) ?
        $form_state['values']['main_container']['members'][$key]['email'] : '';
      $email_len = strlen($email);
      $is_empty = $name_len == 0 && $email_len == 0;
      
      if (!$is_empty) {
                
        $result = gift_exchange_create_member($name, $email, $party);
        if ($result) {
          $num_members++;
        }
      }
    }
  }

  if ($num_members > 0) {
    // Members added.
    drupal_set_message(
      format_plural($num_members, '1 member added.', '@count members added.'));

    ctools_include('ajax');
    ctools_add_js('ajax-responder');
    $path = 'gift-exchange/my-parties/' . $party->nid;
    $options = array(
      'fragment' => 'members_container',
    );
    $commands[] = ctools_ajax_command_redirect($path, 0, $options);
    print ajax_render($commands);
    exit;
  }
  else {
    drupal_set_message(t('No members added.'));

    return $form['main_container']['members'];
  }
}

/**
 * Add Exceptions form.
 *
 * @return object
 *   Import form.
 */
function gift_exchange_add_exceptions_form($form, &$form_state, $node) {

  // Load functions.
  module_load_include('inc', 'gift_exchange', 'includes/gift_exchange.functions');

  // Get member count and redirect if less than 3.
  if (gift_exchange_get_party_members_count($node->nid) < 3) {
    drupal_set_message(t('Party needs at least 3 members.'));
    drupal_goto('gift-exchange/my-parties/' . $node->nid);
  }

  // Set page title.
  drupal_set_title(t('Add exceptions') . ' -> ' . $node->title);

  // Breadcrumb.
  $breadcrumb = array();
  $breadcrumb[] = l(t('My Parties'), 'gift-exchange/my-parties');
  $breadcrumb[] = l($node->title, 'gift-exchange/my-parties/' . $node->nid);
  $breadcrumb[] = t('Add Exceptions');
  gift_exchange_breadcrumb($breadcrumb);

    // Getting node id for validate and submit.
  $form_state['party_nid'] = $node->nid;

  // Main container.
  $form['main_container'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'main_container',
      'class' => array('main-container'),
    ),
  );

  // Actions container.
  $form['main_container']['actions'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'actions_container',
      'class' => array('actions-container', 'clearfix'),
    ),
  );

  $form['main_container']['actions']['view'] = array(
    '#type' => 'html_tag',
    '#tag' => 'div',
    '#value' => l(t('View party'), 'gift-exchange/my-parties/' . $node->nid,
      array(
        'attributes' => array(
          'class' => array('action', 'view-as-admin'),
        )
      )
    ),
    '#attributes' => array(
      'class' => array('action-container', 'view-as-admin-container'),
    ),
  );

  // Info container.
  $form['main_container']['info'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'info_container',
      'class' => array('info-container', 'clearfix'),
    ),
  );

  // Info header.
  $form['main_container']['info']['header'] = array(
    '#type' => 'html_tag',
    '#value' => t('<em>Member</em> A will not give a present to <em>Member B</em>.
        Start typing the names.'),
    '#tag' => 'h3',
    '#attributes' => array('class' => array('header', 'info')),
  );

  // Because we have many fields with the same values, we have to set
  // #tree to be able to access them.
  $form['#tree'] = TRUE;
  $form['main_container']['exceptions'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'id' => 'exceptions_container',
      'class' => array('exceptions-container', 'clearfix'),
    ),
  );

  // Build the fieldset with the proper number of exceptions. We'll use
  // $form_state['num_exceptions'] to determine the number of textfields to build.
  if (empty($form_state['num_exceptions'])) {
    $form_state['num_exceptions'] = 1;
  }
  for ($i = 0; $i < $form_state['num_exceptions']; $i++) {
    // Get if even or odd.
    $row_class = $i % 2 == 0 ? 'even' : 'odd';

    $form['main_container']['exceptions']['exception_' . $i] = array(
      '#type' => 'container',
      '#attributes' => array(
        'id' => 'exception_container',
        'class' => array('exception-container', 'clearfix', $row_class),
      ),
    );
    $form['main_container']['exceptions']['exception_' . $i]['giver'] = array(
      '#type' => 'textfield',
      '#title' => t('!num: Member A', array('!num' => $i+1)),
      '#autocomplete_path' => 'gift-exchange/my-parties/' . $node->nid . '/add-exceptions/autocomplete',
    );
    $form['main_container']['exceptions']['exception_' . $i]['receiver'] = array(
      '#type' => 'textfield',
      '#title' => t('!num: Member B', array('!num' => $i+1)),
      '#autocomplete_path' => 'gift-exchange/my-parties/' . $node->nid . '/add-exceptions/autocomplete',
    );
  }

  // Actions container.
  $form['actions'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'class' => array('actions', 'clearfix'),
    ),
  );

  // Add exception.
  $form['main_container']['add_exception'] = array(
    '#type' => 'submit',
    '#value' => t('Add one more'),
    '#submit' => array('_gift_exchange_add_exceptions_form_add_one_submit'),
    '#ajax' => array(
      'callback' => '_gift_exchange_add_exceptions_form_add_one_callback',
      'wrapper' => 'exceptions_container',
    ),
    '#attributes' => array(
      'class' => array('add-button'),
    ),
  );

  // Back.
  $form['actions']['back'] = array(
    '#type' => 'html_tag',
    '#value' => l(t('Back'), 'gift-exchange/my-parties/' . $node->nid,
      array(
        'attributes' => array(
          'class' => array('action', 'back'),
        ),
        'fragment' => 'exceptions_container',
      )
    ),
    '#tag' => 'div',
    '#attributes' => array(
      'class' => array('back-container', 'clearfix')
    ),
  );

  // Submit.
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#ajax' => array(
      'callback' => '_gift_exchange_add_exceptions_form_callback',
      'wrapper' => 'exceptions_container',
    ),
    '#validate' => array('_gift_exchange_add_exceptions_form_validate_callback'),
    '#attributes' => array(
      'class' => array('save-button'),
    ),
  );

  return $form;
}

/**
 * Autocomplete helper function for Exceptions.
 *
 * @param $node
 *   Gift Exchange Party.
 * @param $string
 *   String that passes.
 */
function _gift_exchange_add_exceptions_autocomplete($node, $string) {
  $matches = array();
  $query = db_select('node', 'm')
    ->fields('m', array('nid', 'title'))
    ->condition('m.type', 'gift_exchange_member')
    ->condition('m.title', '%' . db_like($string) . '%', 'LIKE')
    ->orderBy('m.title', 'ASC')
    ->range(0, 20);

  // Member of the party.
  $query->join('field_data_field_gift_exchange', 'gem',
    'gem.entity_id = m.nid AND gem.field_gift_exchange_target_id = ' . $node->nid);

  $result = $query->execute();

  // Save result to matches
  foreach ($result as $item) {
    $matches[$item->title . ' (' . $item->nid . ')'] = $item->title;
  }

  // Return the result to the form in json.
  drupal_json_output($matches);
}

/**
 * Callback for both ajax-enabled buttons.
 *
 * Selects and returns the container.
 */
function _gift_exchange_add_exceptions_form_add_one_callback($form, &$form_state) {
  return $form['main_container']['exceptions'];
}

/**
 * Submit handler for the "add-one-more" button.
 *
 * Increments the max counter and causes a rebuild.
 */
function _gift_exchange_add_exceptions_form_add_one_submit($form, &$form_state) {
  $form_state['num_exceptions']++;
  $form_state['rebuild'] = TRUE;
}

/**
 * Add Exceptions validate form.
 */
function _gift_exchange_add_exceptions_form_validate_callback($form, &$form_state) {

  // Check names/emails are correct.
  if (isset($form_state['values']['main_container']['exceptions'])) {
    foreach ($form_state['values']['main_container']['exceptions'] as $key => $member) {
      // Check names are mutually filled.
      $filled = $form_state['values']['main_container']['exceptions'][$key]['giver'] != ''
        && $form_state['values']['main_container']['exceptions'][$key]['receiver'] != '';
      $empty = strlen($form_state['values']['main_container']['exceptions'][$key]['giver']) == 0
        && strlen($form_state['values']['main_container']['exceptions'][$key]['receiver']) == 0;

      if (!$filled && !$empty) {
        form_set_error('main_container][exceptions][' . $key . '][giver',
          t('Both names must be filled at %title.',
            array('%title' => $form['main_container']['exceptions'][$key]['giver']['#title'] . ' - ' .
              $form['main_container']['exceptions'][$key]['receiver']['#title'])));
        form_set_error('main_container][exceptions][' . $key . '][receiver');
      }

      // Check exceptions are valid.
      if (!$empty) {
        $party_nid = $form_state['party_nid'];
        module_load_include('inc', 'gift_exchange', 'includes/gift_exchange.functions');

        // Check giver.
        $matches = array();
        preg_match('/.* \(([0-9]*)\)/', $form_state['values']['main_container']['exceptions'][$key]['giver'], $matches);
        $giver_nid = isset($matches[1]) ? $matches[1] : 0;
        if (!gift_exchange_get_member_is_in_party($giver_nid, $party_nid)) {
          form_set_error('main_container][exceptions][' . $key . '][giver',
            t('%title is not a member of the party.',
              array('%title' => $form['main_container']['exceptions'][$key]['giver']['#title'])));
        }

        // Check receiver.
        $matches = array();
        preg_match('/.* \(([0-9]*)\)/', $form_state['values']['main_container']['exceptions'][$key]['receiver'], $matches);
        $receiver_nid = isset($matches[1]) ? $matches[1] : 0;
        if (!gift_exchange_get_member_is_in_party($receiver_nid, $party_nid)) {
          form_set_error('main_container][exceptions][' . $key . '][receiver',
            t('%title is not a member of the party.',
              array('%title' => $form['main_container']['exceptions'][$key]['receiver']['#title'])));
        }

        // Check if exception already exist.
        if (gift_exchange_get_exception($giver_nid, $receiver_nid)) {
          form_set_error('main_container][exceptions][' . $key . '][giver',
            t('Exception already exists at %title.',
              array('%title' => $form['main_container']['exceptions'][$key]['giver']['#title'] . ' - ' .
                $form['main_container']['exceptions'][$key]['receiver']['#title'])));
          form_set_error('main_container][exceptions][' . $key . '][receiver');
        }
      }
    }
  }
}

/**
 * Add Exceptions ajax callback.
 */
function _gift_exchange_add_exceptions_form_callback($form, &$form_state) {
  if (form_get_errors()) {
    $form_state['rebuild'] = TRUE;
    return $form['main_container']['exceptions'];
  }

  // Add exceptions submit form.
  $response = _gift_exchange_add_exceptions_form_submit($form, $form_state);
  return $response;
}

/**
 * Add Exceptions submit form.
 */
function _gift_exchange_add_exceptions_form_submit($form, &$form_state) {

  // Exceptions that will be added.
  $num_exceptions = 0;

  // Create exceptions.
  if (isset($form_state['values']['main_container']['exceptions'])) {
    foreach ($form_state['values']['main_container']['exceptions'] as $key => $exception) {
      // Check names and emails are mutually filled.
      $filled = $form_state['values']['main_container']['exceptions'][$key]['giver'] != ''
        && $form_state['values']['main_container']['exceptions'][$key]['receiver'] != '';
      $empty = strlen($form_state['values']['main_container']['exceptions'][$key]['giver']) == 0
        && strlen($form_state['values']['main_container']['exceptions'][$key]['receiver']) == 0;

      if ($filled) {
        $matches = array();
        preg_match('/.* \(([0-9]*)\)/', $form_state['values']['main_container']['exceptions'][$key]['giver'], $matches);
        $giver_nid = isset($matches[1]) ? $matches[1] : 0;

        $matches = array();
        preg_match('/.* \(([0-9]*)\)/', $form_state['values']['main_container']['exceptions'][$key]['receiver'], $matches);
        $receiver_nid = isset($matches[1]) ? $matches[1] : 0;

        $party_nid = $form_state['party_nid'];

        // Create exception.
        module_load_include('inc', 'gift_exchange', 'includes/gift_exchange.functions');
        gift_exchange_create_exception($giver_nid, $receiver_nid, $party_nid);
        $num_exceptions++;
      }
    }
  }

  if ($num_exceptions > 0) {
    // Exceptions added.
    drupal_set_message(
      format_plural($num_exceptions, '1 exception added.', '@count exceptions added.'));

    ctools_include('ajax');
    ctools_add_js('ajax-responder');
    $path = 'gift-exchange/my-parties/' . $party_nid;
    $options = array(
      'fragment' => 'exceptions_container',
    );
    $commands[] = ctools_ajax_command_redirect($path, 0, $options);
    print ajax_render($commands);
    exit;
  }
  else {
    drupal_set_message(t('No exceptions added.'));

    return $form['main_container']['exceptions'];
  }
}

/**
 * Confirm action callback.
 *
 * @param $js
 *   Nojs or ajax.
 * @param $party
 *   Party node.
 * @param $action
 *   Action to perform.
 */
function gift_exchange_confirm_action_callback($js = NULL, $node, $action) {
  if ($js) {
    // Required includes for ctools to work.
    ctools_include('modal');
    ctools_include('ajax');
    ctools_add_js('ajax-responder');
    ctools_modal_add_js();

    $form = drupal_get_form('gift_exchange_confirm_action_form', $node, $action);

    $content = drupal_render($form);

    ctools_modal_render('', $content);
  }
}

/**
 * Confirm action form.
 *
 * @param $form
 * @param $form_state
 *
 * @return array
 *   Form.
 */
function gift_exchange_confirm_action_form($form, &$form_state, $node, $action) {

  $form['#attributes']['id'] = 'gift_exchange_confirm_form';
  $form['#attributes']['class'][] = 'gift-exchange-confirm-form';

  // Info.
  $form['info'] = array(
    '#type' => 'container',
    '#weight' => 1,
    '#attributes' => array(
      'class' => array('info'),
    ),
  );

  // Header
  $form['info']['header'] = array(
    '#type' => 'html_tag',
    '#tag' => 'div',
    '#value' => t('Are you sure you want to :action party?', array(':action' => $action)),
    '#attributes' => array(
      'class' => array('header'),
    ),
  );

  // Actions.
  $form['actions'] = array(
    '#type' => 'container',
    '#weight' => 1,
    '#attributes' => array(
      'class' => array('actions'),
    ),
  );

  // Yes.
  $form['actions']['yes'] = array(
    '#type' => 'html_tag',
    '#tag' => 'span',
    '#value' => l(t('Yes'), 'gift-exchange/my-parties/' . $node->nid . '/' . $action),
    '#attributes' => array(
      'class' => array('yes'),
    ),
  );

  // Cancel.
  $form['actions']['cancel'] = array(
    '#type' => 'html_tag',
    '#tag' => 'span',
    '#value' => l(t('Cancel'), '#',
      array(
        'attributes' => array(
          'class' => array('ctools-close-modal'),
        )
      )
    ),
    '#attributes' => array(
      'class' => array('cancel'),
    ),
  );

  return $form;
}

/**
 * Confirm delete callback.
 *
 * @param $type
 *   Type of node to delete. Not machine name.
 * @param $js
 *   Nojs or ajax.
 * @param $party
 *   Party node.
 * @param $action
 *   Action to perform.
 */
function gift_exchange_confirm_delete_callback($type, $js = NULL, $node, $action) {
  if ($js) {
    // Required includes for ctools to work.
    ctools_include('modal');
    ctools_include('ajax');
    ctools_add_js('ajax-responder');
    ctools_modal_add_js();

    $form = drupal_get_form('gift_exchange_confirm_action_form', $node, $action);

    // Change yes action.
    $form['actions']['yes'] = array(
      '#type' => 'html_tag',
      '#tag' => 'span',
      '#value' => l(t('Yes'), 'gift-exchange/' . $node->nid . '/delete',
          array(
            'attributes' => array(
              'class' => array('use-ajax'),
            ),
          )
        ),
      '#attributes' => array(
        'class' => array('yes'),
      ),
    );

    $form['info']['header']['#value'] = t('Delete !type?', array('!type' => $type));

    $content = drupal_render($form);

    ctools_modal_render('', $content);
  }
}

/**
 * Delete member or connection.
 *
 * @param $node
 *   Node to delete.
 */
function gift_exchange_delete_callback($node) {
  node_delete($node->nid);

  $result = array(
    '#type' => 'ajax',
    '#commands' => array(
      ajax_command_invoke('.view-ge-members', 'trigger', array('RefreshView')),
      ajax_command_invoke('.view-ge-exceptions', 'trigger', array('RefreshView')),
      ajax_command_invoke('.popups-close', 'click'),
    ),
  );

  return $result;
}

/**
 * Edit member callback.
 *
 * @param $js
 *   Nojs or ajax.
 * @param $member
 *   Member node.
 */
function gift_exchange_edit_member_callback($js = NULL, $member) {
  if ($js) {
    // Required includes for ctools to work.
    ctools_include('modal');
    ctools_include('ajax');
    //ctools_add_js('ajax-responder');
    ctools_modal_add_js();

    module_load_include('inc', 'node', 'node.pages');
    $form = drupal_get_form('gift_exchange_member_node_form', $member);

    //$form['info']['header']['#value'] = t('Edit !member', array('!member' => $member->title));

    $content = drupal_render($form);

    ctools_modal_render(t('Edit !member', array('!member' => $member->title)), $content);
  }
}

/**
 * Update user validate callback.
 */
function _gift_exchange_edit_member_validate($form, &$form_state) {
  module_load_include('inc', 'gift_exchange', 'includes/gift_exchange.functions');
  
  // Check valid mail.
  if (isset($form_state['values']['field_gem_email']['und'][0]['value'])) {
    $email = $form_state['values']['field_gem_email']['und'][0]['value'];
    if (strlen($email) > 0 && !valid_email_address($email)) {
      form_set_error('field_gem_email', t('Invalid email address.'));
    }
  }
  else {
    $email = '';
  }
  
  // Check unique.
  $name = $form_state['values']['title'];
  $member_nid = $form_state['values']['nid'];
  $party_nid = $form_state['values']['field_gift_exchange']['und'][0]['target_id'];
  $member_exists = gift_exchange_get_member_name_email_is_in_party($name, $email, $party_nid, $member_nid);
  
  if ($member_exists) {
    form_set_error('title', t('Member already exists in party.'));
    form_set_error('field_gem_email');
  }
}

/**
 * Submit edit member.
 *
 * @param $form
 * @param $form_state
 *
 * @return array
 *   Action commands.
 */
function _gift_exchange_edit_member_submit($form, &$form_state) {
  module_load_include('inc', 'node', 'node.pages');
  module_load_include('inc', 'gift_exchange', 'includes/gift_exchange.functions');

  ctools_include('modal');
  ctools_include('ajax');
  ctools_modal_add_js();
  ctools_add_js('ajax-responder');

  if (form_get_errors()) {
    $form_state['rebuild'] = TRUE;
    return $form;
  }
  
  $member_nid = $form_state['values']['nid'];
  $name = $form_state['values']['title'];
  if (isset($form_state['values']['field_gem_email']['und'][0]['value'])) {
    $email = $form_state['values']['field_gem_email']['und'][0]['value'];
  }
  else {
    $email = '';
  }
  
  gift_exchange_update_member($member_nid, $name, $email);

  $result = array(
    '#type' => 'ajax',
    '#commands' => array(
      ajax_command_invoke('.view-ge-members', 'trigger', array('RefreshView')),
      ajax_command_invoke('.view-ge-exceptions', 'trigger', array('RefreshView')),
      ajax_command_invoke('.popups-close', 'click'),
    ),
  );

  return $result;
}
